/*
 * Copyright (c) 2021 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef JOB_MANAGER_HH_
#define JOB_MANAGER_HH_

#include <list>
#include <memory>
#include <string>

#include "job.hh"
#include "job-handler.hh"

namespace tizen_cpp {

class JobManager : public Job::IEvent {
 public:
  void AddJobHandler(JobHandler* handler);
  void RemoveJobHandler(JobHandler* handler);
  bool ExistJobHandler(JobHandler* handler);
  void Do(std::shared_ptr<Job> job);
  int Finish(const std::string& job_id);
  void FinishAllJobs();
  void FlushPendingJob(const std::string& job_id);

 private:
  bool ExistJobHandler(const std::string& job_id);
  void AddPendingJob(std::shared_ptr<Job> job);
  void RemovePendingJob(const std::string& job_id);
  void AddRunningJob(std::shared_ptr<Job> job);
  void RemoveRunningJob(const std::string& job_id);

  void OnTimedOut(const Job* job) override;
  void OnRun(const Job* job) override;

 private:
  std::list<JobHandler*> handlers_;
  std::list<std::shared_ptr<Job>> pending_jobs_;
  std::list<std::shared_ptr<Job>> running_jobs_;
};

}  // namespace tizen_cpp

#endif  // JOB_MANAGER_HH_
