/*
 * Copyright (c) 2021 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef JOB_HH_
#define JOB_HH_

#include <bundle_cpp.h>
#include <glib.h>

#include <string>

namespace tizen_cpp {

class Job {
 public:
  class IEvent {
   public:
    virtual void OnTimedOut(const Job* job) = 0;
    virtual void OnRun(const Job* job) = 0;
  };

  Job(int status, std::string id, tizen_base::Bundle data);
  virtual ~Job();

  int GetStatus() const;
  const std::string& GetId() const;
  bundle* GetData() const;
  void SetEventListener(IEvent* listener);
  void SetTimer();
  void UnsetTimer();
  void SetIdler();
  void UnsetIdler();

 private:
  static gboolean TimedOutCb(gpointer data);
  static gboolean IdleCb(gpointer data);

 private:
  int status_;
  std::string id_;
  tizen_base::Bundle data_;
  IEvent* listener_ = nullptr;
  guint timer_ = 0;
  guint idler_ = 0;
};

}  // namespace tizen_cpp

#endif  // JOB_HH_
